@@ -28,8 +28,8 @@
 /* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
 /*************************************************************************/
 
-#ifndef GLOBALS_LIST_H
-#define GLOBALS_LIST_H
+#ifndef LIST_H
+#define LIST_H
 
 #include "core/error_macros.h"
 #include "core/os/memory.h"
@@ -697,4 +697,4 @@ public:
 	};
 };
 
-#endif
+#endif // LIST_H
