@@ -49,8 +49,6 @@ class FileAccessUnix : public FileAccess {
 	String path;
 	String path_src;
 
-	static FileAccess *create_libc();
-
 public:
 	static CloseNotificationFunc close_notification_func;
 
@@ -87,5 +85,6 @@ public:
 	virtual ~FileAccessUnix();
 };
 
-#endif
-#endif
+#endif // UNIX_ENABLED || LIBC_FILEIO_ENABLED
+
+#endif // FILE_ACCESS_UNIX_H
