@@ -126,5 +126,6 @@ public:
 	AudioDriverWASAPI();
 };
 
+#endif // WASAPI_ENABLED
+
 #endif // AUDIO_DRIVER_WASAPI_H
-#endif
