@@ -165,4 +165,4 @@ public:
 	~ShaderEditorPlugin();
 };
 
-#endif
+#endif // SHADER_EDITOR_PLUGIN_H
