@@ -244,4 +244,5 @@ public:
 private:
 	void setup(real_t p_length, bool p_slips_on_slope);
 };
-#endif
+
+#endif // SHAPE_BULLET_H
