@@ -28,8 +28,8 @@
 /* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
 /*************************************************************************/
 
-#ifndef GDMONOFIELD_H
-#define GDMONOFIELD_H
+#ifndef GD_MONO_FIELD_H
+#define GD_MONO_FIELD_H
 
 #include "gd_mono.h"
 #include "gd_mono_header.h"
@@ -74,4 +74,4 @@ public:
 	~GDMonoField();
 };
 
-#endif // GDMONOFIELD_H
+#endif // GD_MONO_FIELD_H
