@@ -38,4 +38,4 @@ void init_thread_jandroid(JavaVM *p_jvm, JNIEnv *p_env);
 void setup_android_thread();
 JNIEnv *get_jni_env();
 
-#endif
+#endif // THREAD_JANDROID_H
