@@ -28,8 +28,8 @@
 /* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
 /*************************************************************************/
 
-#ifndef BACKBUFFERCOPY_H
-#define BACKBUFFERCOPY_H
+#ifndef BACK_BUFFER_COPY_H
+#define BACK_BUFFER_COPY_H
 
 #include "scene/2d/node_2d.h"
 
@@ -71,4 +71,4 @@ public:
 
 VARIANT_ENUM_CAST(BackBufferCopy::CopyMode);
 
-#endif // BACKBUFFERCOPY_H
+#endif // BACK_BUFFER_COPY_H
