@@ -67,7 +67,7 @@ bool Listener::_get(const StringName &p_name, Variant &r_ret) const {
 }
 
 void Listener::_get_property_list(List<PropertyInfo> *p_list) const {
-	p_list->push_back(PropertyInfo(Variant::BOOL, "current"));
+	p_list->push_back(PropertyInfo(Variant::BOOL, PNAME("current")));
 }
 
 void Listener::_update_listener() {
