@@ -149,4 +149,4 @@ public:
 	ButtonGroup();
 };
 
-#endif
+#endif // BASE_BUTTON_H
