@@ -214,7 +214,6 @@ public:
 	int surface_get_array_index_len(int p_idx) const;
 	uint32_t surface_get_format(int p_idx) const;
 	PrimitiveType surface_get_primitive_type(int p_idx) const;
-	bool surface_is_alpha_sorting_enabled(int p_idx) const;
 
 	virtual void surface_set_material(int p_idx, const Ref<Material> &p_material);
 	virtual Ref<Material> surface_get_material(int p_idx) const;
@@ -248,4 +247,4 @@ VARIANT_ENUM_CAST(Mesh::ArrayFormat);
 VARIANT_ENUM_CAST(Mesh::PrimitiveType);
 VARIANT_ENUM_CAST(Mesh::BlendShapeMode);
 
-#endif
+#endif // MESH_H
