@@ -28,8 +28,8 @@
 /* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
 /*************************************************************************/
 
-#ifndef VISUALSERVERVIEWPORT_H
-#define VISUALSERVERVIEWPORT_H
+#ifndef VISUAL_SERVER_VIEWPORT_H
+#define VISUAL_SERVER_VIEWPORT_H
 
 #include "core/self_list.h"
 #include "rasterizer.h"
@@ -190,6 +190,7 @@ public:
 	void viewport_set_use_debanding(RID p_viewport, bool p_debanding);
 	void viewport_set_sharpen_intensity(RID p_viewport, float p_intensity);
 	void viewport_set_hdr(RID p_viewport, bool p_enabled);
+	void viewport_set_use_32_bpc_depth(RID p_viewport, bool p_enabled);
 	void viewport_set_usage(RID p_viewport, VS::ViewportUsage p_usage);
 
 	virtual int viewport_get_render_info(RID p_viewport, VS::ViewportRenderInfo p_info);
@@ -204,4 +205,4 @@ public:
 	virtual ~VisualServerViewport() {}
 };
 
-#endif // VISUALSERVERVIEWPORT_H
+#endif // VISUAL_SERVER_VIEWPORT_H
